/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bresenhamapplet;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.Vector;
import javax.swing.JPanel;

/**
 *
 * @author Sunshine2k
 */
public class BresenhamPanel extends JPanel implements BresenhamEventListener {

    private static final Color backgroundColor = new Color(0xFF, 0xCC, 0x66);
    private static final Color gridColor = new Color(230, 230, 230);

    /**
     * color used for drawing line using DDA algorithm
     */
    public static final Color LineColorDDA          = Color.GRAY;
    /**
     * color used for drawing line using Bresenham algorithm with floating point number
     */
    public static final Color LineColorBHFloat      = Color.BLUE;
    /**
     * color used for drawing line using Bresenham algorithm with integer number
     */
    public static final Color LineColorBHInt        = new Color(0, 153, 0);

    /**
     * color used for drawing circle using simple algorithm and 2-way symmetry
     */
    public static final Color CircleColorSimpleBad  = Color.RED;
    /**
     * color used for drawing circle using simple algorithm and 8-way symmetry
     */
    public static final Color CircleColorSimpleGood = Color.GRAY;
    /**
     * color used for drawing circle using Bresenham algorithm with floating point number
     */
    public static final Color CircleColorBHFloat    = Color.BLUE;
    /**
     * color used for drawing circle using Bresenham algorithm with integer number
     */
    public static final Color CircleColorBHInt      = new Color(0, 153, 0);

    /**
     * color used for drawing circle using simple algorithm and 2-way symmetry
     */
    public static final Color EllipseColorBH        = Color.BLUE;
    
    /**
     * Type of selected painting primitive
     */
    public enum PrimitiveMode {
        LINE,
        CIRCLE,
        ELLIPSE
    }
    // currently selected primitive
    private PrimitiveMode primitiveMode = PrimitiveMode.LINE;

    /**
     * Type of selected line drawing algorithm
     */
    public enum LineDrawingMode {

        /**
         * 
         */
        DDA, BRESENHAMFLOAT, BRESENHAMINT
    }
    private LineDrawingMode lineDrawMode = LineDrawingMode.DDA;

    /**
     * Type of selected circle drawing algorithm
     */
    public enum CircleDrawingMode { SIMPLEBAD, SIMPLEGOOD, BRESENHAMFLOAT, BRESENHAMINT }
    private CircleDrawingMode circleDrawMode = CircleDrawingMode.SIMPLEGOOD;
    
    // if zoom is enabled, this holds the zoom factor
    // improvements: zoomfactor is 1,2,4,8.. so use bit-shifting
    private int zoomMultiplier;

    // current actual position of mouse
    private int mouseX, mouseY;
    // current tile position of mouse
    private int mouseTileX, mouseTileY;

    private MovablePoint p1, p2, p3;

    private Vector<BresenhamEventListener> BresenhamEventListeners = new Vector<BresenhamEventListener>();

    /**
     * Constructor of panel.
     * @param width width of the bresenham panel
     * @param height height of the bresenham panel
     */
    public BresenhamPanel(int width, int height)
    {
        this.setLayout(null);
        this.setBackground(backgroundColor);

        zoomMultiplier = 1;
        
        p1 = new MovablePoint(this, (int)(width * 0.35), height >> 1);
        p2 = new MovablePoint(this, (int)(width * 0.65), (int)(height * 0.6));
        p3 = new MovablePoint(this, (int)(width * 0.5), (int)(height * 0.3));

        // add listeners to both points to get informed about the current position
        // while a point is dragged
        p1.addMovablePointListener(this);
        p2.addMovablePointListener(this);
        p3.addMovablePointListener(this);

        this.addMouseMotionListener(new MouseMotionAdapter()
        {
            @Override
            public void mouseMoved(MouseEvent e)
            {
                mouseX = e.getX();
                mouseY = e.getY();
                mouseTileX = mouseX / zoomMultiplier;
                mouseTileY = mouseY / zoomMultiplier;
                // inform panel about position of cursor
                informListener(mouseTileX, mouseTileY);
                repaint();
                
            }
      
        });
    }

    /**
     * Called every time if a movable point is currently dragged.
     * This event is handled as we the cursor is normally moved over the panel.
     * @param bpe reference to the mouse event
     */
    public void BresenhamEvent(BresenhamPaneMouselEvent bpe) {
        mouseX = bpe.mouseTileX;
        mouseY = bpe.mouseTileY;
        mouseTileX = mouseX / zoomMultiplier;
        mouseTileY = mouseY / zoomMultiplier;
        informListener(mouseTileX, mouseTileY);
        repaint();
    }

    /**
     * Paint our panel - here goes all painting.
     */
    @Override
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);

        // draw grid if necessary
        if (zoomMultiplier != 1)
        {
            g.setColor(gridColor);
            // draw horizontal lines
            for (int y = 0; y < this.getHeight(); y += zoomMultiplier)
            {
                g.drawLine(0, y, this.getWidth(), y);
            }
            // draw vertical lines
            for (int x = 0; x < this.getWidth(); x += zoomMultiplier)
            {
                g.drawLine(x, 0, x, this.getHeight());
            }

            // draw current mouse tile
            g.fillRect( Math.round(mouseX) * zoomMultiplier, Math.round(mouseY) * zoomMultiplier, zoomMultiplier, zoomMultiplier);
        }

        switch (primitiveMode)
        {
            case LINE:
            {
                switch (lineDrawMode)
                {
                    case DDA:
                    {
                        g.setColor(LineColorDDA);
                        LineAlgorithms.drawLineDDA(g, p1.getMidX(), p1.getMidY(),
                                    p2.getMidX(), p2.getMidY(), zoomMultiplier);
                    }
                    break;

                    case BRESENHAMFLOAT:
                    {
                        g.setColor(LineColorBHFloat);
                        LineAlgorithms.drawLineBresenhamFloat(g, p1.getMidX(), p1.getMidY(),
                                    p2.getMidX(), p2.getMidY(), zoomMultiplier);
                    }
                    break;

                    case BRESENHAMINT:
                    {
                        g.setColor(LineColorBHInt);
                        LineAlgorithms.drawLineBresenhamInt(g, p1.getMidX(), p1.getMidY(),
                                    p2.getMidX(), p2.getMidY(), zoomMultiplier);
                    }
                    break;

                    default:
                        // invalid line mode - cannot happen
                        break;
                }
            }
            break;

            case CIRCLE:
            {
                // when drawing a circle, use the first point as midpoint
                // and the distance to seond point as radius
                int radius = (int)Math.sqrt( (p2.getMidX() - p1.getMidX()) * (p2.getMidX() - p1.getMidX()) +
                                             (p2.getMidY() - p1.getMidY()) * (p2.getMidY() - p1.getMidY()));
                switch (circleDrawMode)
                {
                    case SIMPLEBAD:
                    {
                        g.setColor(CircleColorSimpleBad);
                        CircleAlgorithms.drawCircleSimple1(g, p1.getMidX(), p1.getMidY(), radius, zoomMultiplier);
                    }
                    break;

                    case SIMPLEGOOD:
                    {
                        g.setColor(CircleColorSimpleGood);
                        CircleAlgorithms.drawCircleSimple2(g, p1.getMidX(), p1.getMidY(), radius, zoomMultiplier);
                    }
                    break;

                    case BRESENHAMFLOAT:
                    {
                        g.setColor(CircleColorBHFloat);
                        CircleAlgorithms.drawCircleBresenhamFloat(g, p1.getMidX(), p1.getMidY(), radius, zoomMultiplier);
                    }
                    break;

                    case BRESENHAMINT:
                    {
                        g.setColor(CircleColorBHInt);
                        CircleAlgorithms.drawCircleBresenhamInt(g, p1.getMidX(), p1.getMidY(), radius, zoomMultiplier);
                    }
                    break;
                }
            }
            break;

            case ELLIPSE:
            {
                // only bresenham implemented
                int radX = p2.getMidX() - p1.getMidX();
                if (radX < 0)
                    radX *= -1;
                int radY = p3.getMidY() - p1.getMidY();
                if (radY < 0)
                    radY *= -1;
                
                g.setColor(EllipseColorBH);
                EllipseAlgorithms.drawEllipseBresenham(g, p1.getMidX(), p1.getMidY(), radX, radY, zoomMultiplier);
            }
            break;

            default:
                // invalid primitive selected - cannot happen
                break;
        }        
    }

    /**
     * Apply new zoom factor for drawing.
     * @param zoom zoom factor, must be >= 1 and <= 64 and a multiple of 2
     * @return returns true if zoom was succesfully set.
     */
    public boolean setZoomMode(int zoom)
    {
        if ( (zoom >= 1) || (zoom <= 64) )
        {
            zoomMultiplier = zoom;
            this.repaint();
            return true;
        }
        else
            return false;
    }

    /**
     * Set the actual primitive to be drawn. Can only be circle or line primitive.
     * @param mode The new mode to be applied about which primitive to be drawn.
     */
    public void setPrimitiveMode(PrimitiveMode mode)
    {
        primitiveMode = mode;
        p3.setVisible(mode == PrimitiveMode.ELLIPSE);
        repaint();
    }

    /**
     * Set the new algorithm that should be used for drawing the line.
     * @param mode the new mode that should be immediately applied for drawing lines.
     */
    public void setLineDrawingMode(LineDrawingMode mode)
    {
        lineDrawMode = mode;
        repaint();
    }

    /**
     * Set the new algorithm that should be used for drawing a circle.
     * @param mode the new mode that should be immediately applied for
     * drawing circles.
     */
    public void setCircleDrawingMode(CircleDrawingMode mode)
    {
        circleDrawMode = mode;
        repaint();
    }


    /************************************************************
     * Internal event handling
     ************************************************************/

    /**
     * Register to mouseevents to get notified when the current cursor
     * position changes.
     * @param bel The interface that should get be notified & called.
     */
    public void addBresenhamEventListener(BresenhamEventListener bel)
    {
        if (BresenhamEventListeners.contains(bel))
            return;
        BresenhamEventListeners.addElement(bel);
    }

    /**
     * Internal function to inform all registrered objects about new mouse event
     * on panel.
     * @param mtx the x-coordinate of the new mouse position
     * @param mty the y-coordinate of the new mouse position
     */
    private void informListener(int mtx, int mty)
    {
        BresenhamPaneMouselEvent bpe = new BresenhamPaneMouselEvent (this, mtx, mty);
        Vector vtemp = (Vector)BresenhamEventListeners.clone();
        for (int x = 0; x < vtemp.size(); x++)
        {
            BresenhamEventListener target = null;
            target = (BresenhamEventListener)vtemp.elementAt(x);
            target.BresenhamEvent(bpe);
        }
    }

   

}
