/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bresenhamapplet;

import java.awt.Graphics;

/**
 * This class contains the algorithms for rasterizing circles.
 * @author Sunshine2k
 */
public class CircleAlgorithms {

    /**
     * Plots a circle point to graphics object, zoomed.
     * @param g Graphics object used for actual pixel plotting
     * @param x x-coordinate of pixel
     * @param y y-coordinate of pixel
     * @param size size of point to draw; use 1 for normal plotting
     */
    private static void setPixel(Graphics g, int x, int y, int size)
    {
        g.fillRect(x, y, size, size);
    }

    /**
     * Helper function to draw all 8 symmetric circle points.
     * @param g Graphics object used for actual pixel plotting
     * @param mx x-coordinate of center of circle
     * @param my y-coordinate of center of circle
     * @param x x-value for point on circle from center
     * @param y y-value for point on circle from center
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    private static void drawSymmetricPoints(Graphics g, int mx, int my, int x, int y, int multiplier)
    {
        setPixel(g, (mx + x) * multiplier, (my - y) * multiplier, multiplier);
        setPixel(g, (mx + x) * multiplier, (my + y) * multiplier, multiplier);
        setPixel(g, (mx - x) * multiplier, (my + y) * multiplier, multiplier);
        setPixel(g, (mx - x) * multiplier, (my - y) * multiplier, multiplier);
        setPixel(g, (mx + y) * multiplier, (my - x) * multiplier, multiplier);
        setPixel(g, (mx + y) * multiplier, (my + x) * multiplier, multiplier);
        setPixel(g, (mx - y) * multiplier, (my + x) * multiplier, multiplier);
        setPixel(g, (mx - y) * multiplier, (my - x) * multiplier, multiplier);
    }

    
    /***************************************************************************
     * Public drawing functions
     ***************************************************************************/

    /**
     * Simple approach evaluating the circle equation
     * y = y0 +- sqrt(r^2 - (x-x0)^2)
     * and make only use of 2-way symmetry.
     * @param g Graphics object where circle is plotted
     * @param midPointX x-coordinate of center of circle
     * @param midPointY y-coordinate of center of circle
     * @param radius radius of circle
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawCircleSimple1(Graphics g, int midPointX, int midPointY, int radius, int multiplier)
    {
        midPointX /= multiplier;
        midPointY /= multiplier;
        radius /= multiplier;
        
        int rSquare = radius * radius;
        double fy;
        for (int x = -radius; x < radius; x++)
        {
            fy = Math.sqrt(rSquare - (x*x));
            setPixel(g, Math.round(midPointX + x) * multiplier, (int)Math.round(midPointY + fy) * multiplier, multiplier);
            setPixel(g, Math.round(midPointX + x) * multiplier, (int)Math.round(midPointY - fy) * multiplier, multiplier);
        }
    }

    /**
     * Simple circle rasterising algorithm, also evaluating the circle equation, but
     * makes better use of symmetry.
     * @param g Graphics object where circle is plotted
     * @param midPointX x-coordinate of center of circle
     * @param midPointY y-coordinate of center of circle
     * @param radius radius of circle
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawCircleSimple2(Graphics g, int midPointX, int midPointY, int radius, int multiplier)
    {
        midPointX /= multiplier;
        midPointY /= multiplier;
        radius /= multiplier;
        double y = radius;
        int x = 1;
        int rSquare = radius * radius;
        y = (int) (Math.sqrt(rSquare - x*x) + 0.5);

        setPixel(g, midPointX * multiplier, (midPointY + radius) * multiplier, multiplier);
        setPixel(g, midPointX * multiplier, (midPointY - radius) * multiplier, multiplier);
        setPixel(g, (midPointX - radius) * multiplier, midPointY * multiplier, multiplier);
        setPixel(g, (midPointX + radius) * multiplier, midPointY * multiplier, multiplier);
        while (x <= y)
        {
            setPixel(g, (midPointX + x) * multiplier, (int)(midPointY + y) * multiplier, multiplier);
            setPixel(g, (midPointX + x) * multiplier, (int)(midPointY - y) * multiplier, multiplier);
            setPixel(g, (midPointX - x) * multiplier, (int)(midPointY + y) * multiplier, multiplier);
            setPixel(g, (midPointX - x) * multiplier, (int)(midPointY - y) * multiplier, multiplier);
            setPixel(g, (int)(midPointX + y) * multiplier, (midPointY + x) * multiplier, multiplier);
            setPixel(g, (int)(midPointX + y) * multiplier, (midPointY - x) * multiplier, multiplier);
            setPixel(g, (int)(midPointX - y) * multiplier, (midPointY + x) * multiplier, multiplier);
            setPixel(g, (int)(midPointX - y) * multiplier, (midPointY - x) * multiplier, multiplier);
            x += 1;
            y = (int) (Math.sqrt(rSquare - x*x) + 0.5);
        }
    }

    /**
     * Bresenham circle rasterising algorithm, using floating point numbers.
     * makes better use of symmetry.
     * @param g Graphics object where circle is plotted
     * @param midPointX x-coordinate of center of circle
     * @param midPointY y-coordinate of center of circle
     * @param radius radius of circle
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawCircleBresenhamFloat(Graphics g, int midPointX, int midPointY, int radius, int multiplier)
    {
        midPointX /= multiplier;
        midPointY /= multiplier;
        radius /= multiplier;
        int x = 0;
        int y = radius;
        double d = 1.25 - radius;

        drawSymmetricPoints(g, midPointX, midPointY, x, y, multiplier);
        while (x < y)
        {
            if (d < 0)
            {
                d = d + 2 * x + 3;
                x++;
            }
            else
            {
                d = d + 2 * (x-y) + 5;
                x++;
                y--;
            }
            drawSymmetricPoints(g, midPointX, midPointY, x, y, multiplier);
        }
    }

    /**
     * Bresenham circle rasterising algorithm, using integer numbers.
     * makes better use of symmetry.
     * @param g Graphics object where circle is plotted
     * @param midPointX x-coordinate of center of circle
     * @param midPointY y-coordinate of center of circle
     * @param radius radius of circle
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawCircleBresenhamInt(Graphics g, int midPointX, int midPointY, int radius, int multiplier)
    {
        midPointX /= multiplier;
        midPointY /= multiplier;
        radius /= multiplier;
        int x = 0;
        int y = radius;
        int d = 1 - radius;

        drawSymmetricPoints(g, midPointX, midPointY, x, y, multiplier);
        while (x < y)
        {
            if (d < 0)
            {
                d = d + 2 * x + 3;
                x++;
            }
            else
            {
                d = d + 2 * (x-y) + 5;
                x++;
                y--;
            }
            drawSymmetricPoints(g, midPointX, midPointY, x, y, multiplier);
        }
    }
}
