/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package bresenhamapplet;

import java.awt.Graphics;

/**
 * This class contains the algorithms for rasterizing lines.
 * @author Sunshine2k
 */
public class LineAlgorithms {

    /**
     * Plots a circle point to graphics object, zoomed.
     * @param g Graphics object used for actual pixel plotting
     * @param x x-coordinate of pixel
     * @param y y-coordinate of pixel
     * @param size size of point to draw; use 1 for normal plotting
     */
    private static void setPixel(Graphics g, int x, int y, int size)
    {
        g.fillRect(x, y, size, size);
    }

    /***************************************************************************
     * Public drawing functions
     ***************************************************************************/

    /**
     * Digitial Differential Analyzer algorithm for rasterising lines.
     * @param g Graphics object where circle is plotted
     * @param x1 x-coordinate of first endpoint
     * @param y1 y-coordinate of first endpoint
     * @param x2 x-coordinate of second endpoint
     * @param y2 y-coordinate of second endpoint
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawLineDDA(Graphics g, int x1, int y1, int x2, int y2, int multiplier)
    {
        int len;
        float dx, dy;
        float x, y;

        x1 /= multiplier;
        x2 /= multiplier;
        y1 /= multiplier;
        y2 /= multiplier;

        // estimate length of line
        if (Math.abs(x2 - x1) >= Math.abs(y2 - y1))
            len = Math.abs(x2 - x1);
        else
            len = Math.abs(y2 - y1);

        // calculate increments
        dx = (float)(x2 - x1) / len;
        dy = (float)(y2 - y1) / len;

        // start point
        x = (int)(x1 + 0.5);
        y = (int)(y1 + 0.5);

        // draw actual line step by step
        for (int i = 0; i < len; i++)
        {
            setPixel(g, (Math.round(x)*multiplier),
                        (Math.round(y)*multiplier), multiplier);
            x += dx;
            y += dy;
        }
        /* draw final endpoint which may be excluded and looks bad in zoom mode */
        setPixel(g, (Math.round(x2)*multiplier),
                    (Math.round(y2)*multiplier), multiplier);
    }

    /**
     * Bresenham line rasterising algorithm, using floating point numbers.
     * @param g Graphics object where circle is plotted
     * @param x1 x-coordinate of first endpoint
     * @param y1 y-coordinate of first endpoint
     * @param x2 x-coordinate of second endpoint
     * @param y2 y-coordinate of second endpoint
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawLineBresenhamFloat(Graphics g, int x1, int y1,
                                                int x2, int y2, int multiplier)
    {
        boolean changed = false;
        x1 /= multiplier;
        x2 /= multiplier;
        y1 /= multiplier;
        y2 /= multiplier;
        int x = x1;
        int y = y1;

        // get estimation of length
        int dx = Math.abs(x2 - x1);
        int dy = Math.abs(y2 - y1);
        // get direction info
        int signx = (int)Math.signum(x2 - x1);
        int signy = (int)Math.signum(y2 - y1);

        if (dy > dx)
        {   // swap values
            int tmp = dx;
            dx = dy;
            dy = tmp;
            changed = true;
        }
        // calculate starting error value
        float e = (float)((float)(dy)/(float)(dx) - 0.5f);

        for (int i = 1; i <= dx; i++)
        {
            setPixel(g, (Math.round(x)*multiplier),
                        (Math.round(y)*multiplier), multiplier);
            while (e >= 0)
            {
                if (changed)
                    x += signx;
                else
                    y += signy;
                e = e - 1;
            }
            if (changed)
                y += signy;
            else
                x += signx;
            e = e + (float)((float)dy/(float)dx);
        }
        /* draw final endpoint which may be excluded and looks bad in zoom mode */
        setPixel(g, (Math.round(x2)*multiplier),
                        (Math.round(y2)*multiplier), multiplier);
    }

    /**
     * Bresenham line rasterising algorithm, using integer numbers.
     * @param g Graphics object where circle is plotted
     * @param x1 x-coordinate of first endpoint
     * @param y1 y-coordinate of first endpoint
     * @param x2 x-coordinate of second endpoint
     * @param y2 y-coordinate of second endpoint
     * @param multiplier zoom multiplier, use 1 for normal drawing
     */
    public static void drawLineBresenhamInt(Graphics g, int x1, int y1,
                                            int x2, int y2, int multiplier)
    {
        boolean changed = false;
        x1 /= multiplier;
        x2 /= multiplier;
        y1 /= multiplier;
        y2 /= multiplier;
        
        int x = x1;
        int y = y1;
        // get estimation of length
        int dx = Math.abs(x2 - x1);
        int dy = Math.abs(y2 - y1);
        // get direction info
        int signx = (int)Math.signum(x2 - x1);
        int signy = (int)Math.signum(y2 - y1);

        if (dy > dx)
        {   // swawp values
            int tmp = dx;
            dx = dy;
            dy = tmp;
            changed = true;
        }
        // starting error value
        int e = 2 * dy - dx;

        for (int i = 1; i <= dx; i++)
        {
            setPixel(g, (Math.round(x)*multiplier),
                        (Math.round(y)*multiplier), multiplier);
            while (e >= 0)
            {
                if (changed)
                    x += signx;
                else
                    y += signy;
                e = e - 2 * dx;
            }
            if (changed)
                y += signy;
            else
                x += signx;
            e = e + 2 * dy;
        }
        /* draw final endpoint which may be excluded and looks bad in zoom mode */
        setPixel(g, (Math.round(x2)*multiplier),
                   (Math.round(y2)*multiplier), multiplier);
    }
}
